import abc


class GettableResourceInterface(abc.ABC):
    @abc.abstractmethod
    def fetch_item(self, code_or_item):
        pass


class ListableResourceInterface(abc.ABC):
    @abc.abstractmethod
    def fetch_list(self, args=None):
        pass


class CreatableResourceInterface(abc.ABC):
    @abc.abstractmethod
    def create_item(self, item):
        pass


class UpdatableResourceInterface(abc.ABC):
    # @abc.abstractmethod
    def update_create_item(self, item_values, code=None):
        pass


class UpdatableListResourceInterface(abc.ABC):
    # @abc.abstractmethod
    def update_create_list(self, item_values, code=None):
        pass


class DeletableResourceInterface(abc.ABC):
    @abc.abstractmethod
    def delete_item(self, code_or_item):
        pass


class CodeBasedResourceInterface(abc.ABC):
    @abc.abstractmethod
    def get_code(self, item):
        pass



import requests
from pyakeneo.auth import Auth
from pyakeneo.resources import (AssetFamilyPool, AssociationTypesPool,
                                AttributeGroupsPool, AttributesPool,
                                CategoriesPool, ChannelsPool, CurrenciesPool,
                                FamiliesPool, LocalesPool, MeasureFamiliesPool,
                                MediaFilesPool, ProductModelsPool,
                                ProductsPool, PublishedProductsPool,
                                ReferenceEntityPool)
from pyakeneo.utils import urljoin


class Client:
    BASIC_API_PATH = "/api/rest/v1/"

    def __init__(
        self,
        base_url,
        client_id=None,
        secret=None,
        username=None,
        password=None,
        session=None,
        auth=None,
    ):
        print(
            base_url,
            client_id,
            secret,
            username,
            password,
            session,
            auth
        )
        """Expect credential
        1) as auth, or
        2) as client_id+secret+username+password, or
        3) as session having an authentication."""
        provided_auth = False
        if not auth:
            if client_id or secret or username or password:
                if client_id and secret and username and password:
                    provided_auth = True
                    auth = Auth(base_url, client_id, secret, username, password)
            elif session:
                provided_auth = True
        else:
            provided_auth = True
        if not provided_auth:
            raise ValueError(
                "Expect credential 1) as auth, or "
                + "2) as client_id+secret+username+password, or "
                + "3) as session having an authentication."
            )
        if not session:
            session = requests.Session()
        self._init(base_url, session, auth)

    def _init(self, base_url, session, auth):
        self._base_url = base_url
        self._session = session
        if auth:
            self._session.auth = auth
        self._session.headers.update({"Content-Type": "application/json"})
        self._resources = {
            "association_types": AssociationTypesPool(
                urljoin(self._base_url, self.BASIC_API_PATH, "association-types/"),
                session,
            ),
            "attributes": AttributesPool(
                urljoin(self._base_url, self.BASIC_API_PATH, "attributes/"), session
            ),
            "attribute_groups": AttributeGroupsPool(
                urljoin(self._base_url, self.BASIC_API_PATH, "attribute-groups/"),
                session,
            ),
            "categories": CategoriesPool(
                urljoin(self._base_url, self.BASIC_API_PATH, "categories/"), session
            ),
            "channels": ChannelsPool(
                urljoin(self._base_url, self.BASIC_API_PATH, "channels/"), session
            ),
            "currencies": CurrenciesPool(
                urljoin(self._base_url, self.BASIC_API_PATH, "currencies/"), session
            ),
            "families": FamiliesPool(
                urljoin(self._base_url, self.BASIC_API_PATH, "families/"), session
            ),
            "locales": LocalesPool(
                urljoin(self._base_url, self.BASIC_API_PATH, "locales/"), session
            ),
            "measure_families": MeasureFamiliesPool(
                urljoin(self._base_url, self.BASIC_API_PATH, "measure-families/"),
                session,
            ),
            "media_files": MediaFilesPool(
                urljoin(self._base_url, self.BASIC_API_PATH, "media-files/"), session
            ),
            "products": ProductsPool(
                urljoin(self._base_url, self.BASIC_API_PATH, "products/"), session
            ),
            "product_models": ProductModelsPool(
                urljoin(self._base_url, self.BASIC_API_PATH, "product-models/"), session
            ),
            "published_products": PublishedProductsPool(
                urljoin(self._base_url, self.BASIC_API_PATH, "published-products/"),
                session,
            ),
            "asset_families": AssetFamilyPool(
                urljoin(self._base_url, self.BASIC_API_PATH, "asset-families/"), session
            ),
            "reference_entities": ReferenceEntityPool(
                urljoin(self._base_url, self.BASIC_API_PATH, "reference-entities/"),
                session,
            ),
        }

    @property
    def resources(self):
        """Return all resources as a list of Resources"""
        return self._resources

    @property
    def association_types(self):
        return self._resources["association_types"]

    @property
    def attributes(self):
        return self._resources["attributes"]

    @property
    def attribute_groups(self):
        return self._resources["attribute_groups"]

    @property
    def categories(self):
        return self._resources["categories"]

    @property
    def channels(self):
        return self._resources["channels"]

    @property
    def currencies(self):
        return self._resources["currencies"]

    @property
    def families(self):
        return self._resources["families"]

    @property
    def locales(self):
        return self._resources["locales"]

    @property
    def measure_families(self):
        return self._resources["measure_families"]

    @property
    def media_files(self):
        return self._resources["media_files"]

    @property
    def products(self):
        return self._resources["products"]

    @property
    def product_models(self):
        return self._resources["product_models"]

    @property
    def published_products(self):
        return self._resources["published_products"]

    @property
    def asset_families(self):
        return self._resources["asset_families"]

    @property
    def reference_entities(self):
        return self._resources["reference_entities"]



# -*- coding: utf-8 -*-

import json


class Result(object):
    """Holds the result of a search. It can be iterated through as a list,
    as an iterator, or as a generator. Note that only one iteration should be
    used on a given Collection object.
    Search results are paginated: https://api.akeneo.com/documentation/pagination.html

    The next page will be loaded once the user
    iterated over the whole current page. The content of the new page will replace
    the content of the previous page."""

    def __init__(self, session, count, current_items, link_first, link_next, link_self):
        self._items = current_items
        self._link_next = link_next
        self._link_self = link_self
        self._link_first = link_first
        self._count = count
        self._session = session
        self._page_iterator = iter(self._items)
        self._reached_the_end = False

    def __iter__(self):
        while not self._reached_the_end:
            for item in self._page_iterator:
                yield item
            self.fetch_next_page()

    def __next__(self):
        try:
            return next(self._page_iterator)
        except Exception:
            self.fetch_next_page()
            if not self._reached_the_end:
                return next(self._page_iterator)
            else:
                return

    def get_page_items(self):
        return self._items

    def fetch_next_page(self):
        """Return True if a next page exists. Returns False otherwise."""
        if self._link_next:
            r = self._session.get(self._link_next)
            if r.status_code == 200:
                (
                    self._link_first,
                    self._link_self,
                    self._link_next,
                    self._items,
                    self._count,
                ) = Result.parse_page(json.loads(r.text))
                self._page_iterator = iter(self._items)
                self._reached_the_end = False
            else:
                self._reached_the_end = True
        else:
            self._reached_the_end = True
        return not self._reached_the_end

    def get_count(self):
        return self._count

    def get_next_link(self):
        return self._link_next

    def get_self_link(self):
        return self._link_self

    def get_first_link(self):
        return self._link_first

    @staticmethod
    def parse_page(json_data):
        """Returns (next link, retrieved items, count of items)"""
        final_next_link = None
        next_link = json_data["_links"].get("next")
        if next_link:
            final_next_link = next_link["href"]
        return (
            json_data["_links"]["first"]["href"],
            json_data["_links"]["self"]["href"],
            final_next_link,
            json_data["_embedded"]["items"],
            json_data.get("items_count"),
        )

    @staticmethod
    def from_json_text(session, json_text):
        json_data = json.loads(json_text)
        parsed = Result.parse_page(json_data)
        return Result(session, parsed[4], parsed[3], parsed[0], parsed[2], parsed[1])



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pyakeneo']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.28.2,<3.0.0', 'structlog>=22.3.0,<23.0.0']

setup_kwargs = {
    'name': 'pyakeneo',
    'version': '0.4.1',
    'description': 'Python client for the Akeneo API REST',
    'long_description': "|Build Status|\n|Documentation Status|\n\n\nPython client for Akeneo PIM API\n================================\n\nA simple Python client to use the `Akeneo PIM API`_.\n\nDependencies are managed with `poetry`_\n(list of dependencies available in `pyproject.toml`_).\n\nYou may install them with:\n\n.. code:: bashasda\n\n        poetry install --dev\n\nInstallation\n------------\n\n.. code:: bash\n\n        poetry install pyakeneo\n        \nUsage\n-----\n\nA simple example is provided in `docs/example_exporter.py`_.\n\nIf you experience issues when importing modules, run the examples as follow:\n\n.. code:: bash\n\n        cd docs\n        poetry run python example_exporter.py\n        \n\n.. _docs/example_exporter.py: https://raw.githubusercontent.com/kavetech/akeneo_api_client/master/docs/example.py\n\nTests\n-----\n\nRun tests as follow:\n\n.. code:: bash\n\n        poetry run nosetests\n        \nIf tests don't pass in your environment, please check that dependencies match those described in pyproject.toml. One way to do it is to ensure that poetry runs commands in a dedicated virtualenv by setting environment variable as follow:\n\n.. code:: bash\n\n        poetry install --dev\n\n\nTests are provided with mocks, recorded with `VCR.py`_. In case you need\nto (re)run tests, you should install the dataset in you PIM instance as\nfollow:\n\n.. _Akeneo PIM API: https://api.akeneo.com/\n.. _poetry: https://github.com/python-poetry/poetry\n.. _VCR.py: http://vcrpy.readthedocs.io/en/latest/index.html\n.. _pyproject.toml: https://python-poetry.org/docs/pyproject/\n\n.. |Build Status| image:: https://travis-ci.org/matthieudelaro/akeneo_api_client.svg?branch=master\n   :target: https://travis-ci.org/matthieudelaro/akeneo_api_client\n.. |Documentation Status| image:: https://readthedocs.org/projects/akeneo-api-client/badge/?version=latest\n   :target: http://akeneo-api-client.readthedocs.io/en/latest/\n",
    'author': 'Kave Tech',
    'author_email': 'kavetech@kavehome.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
